import { EventCounter } from "@/utils/EventCounter";
import { getComponentState } from "@/utils/GetComponentState";
import { stopPropagation } from "@/utils/StopPropagation";
import { Slider, SliderOwnProps, styled } from "@mui/material";
import React from "react";

export type GainSliderOnChangeEventType = "GainChanged";

export interface GainSliderOnChangeEvent {
  type: GainSliderOnChangeEventType;
  gain: number;
}

export interface GainSliderOwnProps {
  gain?: number;
  onChange: (event: GainSliderOnChangeEvent) => any;
}
export interface GainSliderOwnState {
  gainPercent: number;
}
export type GainSliderProps = GainSliderOwnProps& {sliderProps?: SliderOwnProps}
export const SliderStyled = styled(Slider)(({ theme }) => ({
  "& .MuiSlider-thumb": {
    width: "25px",
    height: "25px",
  },
  "& .MuiSlider-valueLabel": {
    top: 10,
    left: -10,
    fontFamily: '"Roboto","Helvetica","Arial",sans-serif',
    fontWeight: 400,
    fontSize: "0.75rem",
    width: "4em",
    letterSpacing: "0.03333em",
    backgroundColor: "unset",
    color: theme.palette.text.primary,
    "&::before": {
      display: "none",
    },
    "& *": {
      background: "transparent",
      color: theme.palette.mode === "dark" ? "#000" : "#fff",
    },
  },
}));

export class GainSlider extends React.Component<
GainSliderProps,
  GainSliderOwnState
> {
  /*[fromRangeMin, fromRangeMax],
    [toRangeMin, toRangeMax]*/
  private static o = {
    //originally gain to percent
    lessThanHalf: [
      [-60, 0],
      [0, 50],
    ],
    halfOrMore: [
      [0, 12],
      [50, 100],
    ],
  };
  static gainToPercent(gain: number) {
    const key: keyof typeof GainSlider.o =
      gain >= 0 ? "halfOrMore" : "lessThanHalf";
    return this.convertRange(gain, ...GainSlider.o[key]);
  }
  static percentToGain(percent: number) {
    const key: keyof typeof GainSlider.o =
      percent >= 50 ? "halfOrMore" : "lessThanHalf";
    return this.convertRange(
      percent,
      ...GainSlider.o[key].map((i) => i.reverse()).reverse()
    );
  }
  static convertRange(value: number, ...ranges: number[][]) {
    const [output, input] = ranges;
    return (
      ((value - output[0]) * (input[1] - input[0])) / (output[1] - output[0]) +
      input[0]
    );
  }
  static stopPropagation(event: Event | React.MouseEvent) {
    if (event instanceof Event) event.stopImmediatePropagation();
    else event.nativeEvent.stopImmediatePropagation();
  }

  private eventCounter = new EventCounter<"SliderClick">();
  defaultValues: GainSliderOwnState = { gainPercent: 50 };
  constructor(props: GainSliderProps) {
    super(props);
    this.state = getComponentState<GainSliderOwnState, "gainPercent">(
      props,
      this.defaultValues
    );
    this.eventCounter.on({
      name: "SliderClick",
      count: 2,
      callback: this.resetGain.bind(this),
    });
  }

  resetGain() {
    this.setState({ gainPercent: this.defaultValues.gainPercent });
  }

  onGainChange(event: Event): void {
    let changedGain = parseFloat((event.target as HTMLInputElement).value);
    if (changedGain > 100) changedGain = 100;
    else if (changedGain < 0) changedGain = 0;
    this.setState({
      gainPercent: changedGain,
    });
    this.props.onChange({
      type: "GainChanged",
      gain: Math.round(GainSlider.percentToGain(changedGain) * 10) / 10,
    });
  }
  onSliderClick(e: React.MouseEvent<HTMLSpanElement, MouseEvent>): void {
    stopPropagation(e);
    this.eventCounter.emit("SliderClick");
  }

  render(): React.ReactNode {
    return (
      <SliderStyled
        sx={{
          // margin: "5px",
          marginInline: "10px",
        }}
        valueLabelFormat={(e) => `${e.toFixed(1)} dB`}
        // orientation="vertical"
        color={
          (this.state.gainPercent < GainSlider.gainToPercent(0) && "info") ||
          (this.state.gainPercent > GainSlider.gainToPercent(0) &&
            this.state.gainPercent < 100 &&
            "warning") ||
          (this.state.gainPercent === 100 && "error") ||
          "primary"
        }
        step={1}
        min={0}
        max={100}
        value={this.state.gainPercent}
        onChange={(e) => this.onGainChange(e)}
        onClick={(e) => this.onSliderClick(e)}
        scale={(v) => GainSlider.percentToGain(v)}
        {...this.props.sliderProps}
      />
    );
  }
}
